@@ -39,6 +39,11 @@
 
 #define MAX_NESTING_LEVEL 4
 
+struct nest_info {
+	uint16_t type;
+	uint16_t offset;
+};
+
 struct genl_unicast_notify {
 	l_genl_msg_func_t handler;
 	l_genl_destroy_func_t destroy;
@@ -60,7 +65,6 @@ struct l_genl {
 	unsigned int next_notify_id;
 	struct l_queue *family_list;
 	struct l_genl_family *nlctrl;
-	struct genl_unicast_notify *unicast_notify;
 	l_genl_debug_func_t debug_callback;
 	l_genl_destroy_func_t debug_destroy;
 	void *debug_data;
@@ -74,7 +78,7 @@ struct l_genl_msg {
 	void *data;
 	uint32_t size;
 	uint32_t len;
-	struct nlattr *nests[MAX_NESTING_LEVEL];
+	struct nest_info nests[MAX_NESTING_LEVEL];
 	uint8_t nesting_level;
 };
 
@@ -124,6 +128,7 @@ struct l_genl_family {
 	l_genl_destroy_func_t watch_destroy;
 	void *watch_data;
 	unsigned int nlctrl_cmd;
+	struct genl_unicast_notify *unicast_notify;
 };
 
 static void destroy_request(void *data)
@@ -251,12 +256,32 @@ static struct l_genl_msg *msg_alloc(uint8_t cmd, uint8_t version, uint32_t size)
 	msg->len = NLMSG_HDRLEN + GENL_HDRLEN;
 	msg->size = msg->len + NLMSG_ALIGN(size);
 
-	msg->data = l_new(unsigned char, msg->size);
+	msg->data = l_realloc(NULL, msg->size);
+	memset(msg->data, 0, msg->size);
 	msg->nesting_level = 0;
 
 	return l_genl_msg_ref(msg);
 }
 
+static bool msg_grow(struct l_genl_msg *msg, uint32_t needed)
+{
+	uint32_t grow_by;
+
+	if (msg->size >= msg->len + needed)
+		return true;
+
+	grow_by = msg->size - needed;
+
+	if (grow_by < 32)
+		grow_by = 128;
+
+	msg->data = l_realloc(msg->data, msg->size + grow_by);
+	memset(msg->data + msg->size, 0, grow_by);
+	msg->size += grow_by;
+
+	return true;
+}
+
 struct l_genl_msg *_genl_msg_create(const struct nlmsghdr *nlmsg)
 {
 	struct l_genl_msg *msg;
@@ -367,7 +392,6 @@ static void process_unicast(struct l_genl *genl, const struct nlmsghdr *nlmsg)
 {
 	struct l_genl_msg *msg;
 	struct genl_request *request;
-	struct genl_unicast_notify *notify;
 
 	if (nlmsg->nlmsg_type == NLMSG_NOOP ||
 					nlmsg->nlmsg_type == NLMSG_OVERRUN)
@@ -400,9 +424,22 @@ static void process_unicast(struct l_genl *genl, const struct nlmsghdr *nlmsg)
 			wakeup_writer(genl);
 		}
 	} else {
-		notify = genl->unicast_notify;
-		if (notify && notify->handler)
-			notify->handler(msg, notify->user_data);
+		const struct l_queue_entry *entry;
+		struct genl_unicast_notify *notify;
+
+		for (entry = l_queue_get_entries(genl->family_list);
+				entry; entry = entry->next) {
+			struct l_genl_family *family = entry->data;
+
+			if (family->id != nlmsg->nlmsg_type)
+				continue;
+
+			notify = family->unicast_notify;
+			if (notify->handler)
+				notify->handler(msg, notify->user_data);
+
+			break;
+		}
 	}
 
 	l_genl_msg_unref(msg);
@@ -624,8 +661,6 @@ LIB_EXPORT void l_genl_unref(struct l_genl *genl)
 	if (genl->debug_destroy)
 		genl->debug_destroy(genl->debug_data);
 
-	l_genl_set_unicast_handler(genl, NULL, NULL, NULL);
-
 	l_free(genl);
 }
 
@@ -657,41 +692,6 @@ LIB_EXPORT bool l_genl_set_close_on_unref(struct l_genl *genl, bool do_close)
 	return true;
 }
 
-LIB_EXPORT bool l_genl_set_unicast_handler(struct l_genl *genl,
-						l_genl_msg_func_t handler,
-						void *user_data,
-						l_genl_destroy_func_t destroy)
-{
-	struct genl_unicast_notify *notify;
-
-	if (!genl)
-		return false;
-
-	notify = genl->unicast_notify;
-	if (notify) {
-		if (notify->destroy)
-			notify->destroy(notify->user_data);
-
-		if (!handler) {
-			l_free(notify);
-			genl->unicast_notify = NULL;
-			return true;
-		}
-	} else {
-		if (!handler)
-			return false;
-
-		notify = l_new(struct genl_unicast_notify, 1);
-		genl->unicast_notify = notify;
-	}
-
-	notify->handler = handler;
-	notify->destroy = destroy;
-	notify->user_data = user_data;
-
-	return true;
-}
-
 const void *_genl_msg_as_bytes(struct l_genl_msg *msg, uint16_t type,
 					uint16_t flags, uint32_t seq,
 					uint32_t pid,
@@ -783,14 +783,16 @@ LIB_EXPORT bool l_genl_msg_append_attr(struct l_genl_msg *msg, uint16_t type,
 	if (unlikely(!msg))
 		return false;
 
-	if (msg->len + NLA_HDRLEN + NLA_ALIGN(len) > msg->size)
+	if (!msg_grow(msg, NLA_HDRLEN + NLA_ALIGN(len)))
 		return false;
 
 	nla = msg->data + msg->len;
 	nla->nla_len = NLA_HDRLEN + len;
 	nla->nla_type = type;
 
-	memcpy(msg->data + msg->len + NLA_HDRLEN, data, len);
+	if (len)
+		memcpy(msg->data + msg->len + NLA_HDRLEN, data, len);
+
 	msg->len += NLA_HDRLEN + NLA_ALIGN(len);
 
 	return true;
@@ -810,7 +812,7 @@ LIB_EXPORT bool l_genl_msg_append_attrv(struct l_genl_msg *msg, uint16_t type,
 	for (i = 0; i < iov_len; i++)
 		len += iov[i].iov_len;
 
-	if (msg->len + NLA_HDRLEN + NLA_ALIGN(len) > msg->size)
+	if (!msg_grow(msg, NLA_HDRLEN + NLA_ALIGN(len)))
 		return false;
 
 	nla = msg->data + msg->len;
@@ -831,22 +833,17 @@ LIB_EXPORT bool l_genl_msg_append_attrv(struct l_genl_msg *msg, uint16_t type,
 
 LIB_EXPORT bool l_genl_msg_enter_nested(struct l_genl_msg *msg, uint16_t type)
 {
-	struct nlattr *nla;
-
 	if (unlikely(!msg))
 		return false;
 
 	if (unlikely(msg->nesting_level == MAX_NESTING_LEVEL))
 		return false;
 
-	if (msg->len + NLA_HDRLEN > msg->size)
+	if (!msg_grow(msg, NLA_HDRLEN))
 		return false;
 
-	nla = msg->data + msg->len;
-	nla->nla_type = type;
-	nla->nla_len = msg->len; /* Save position */
-
-	msg->nests[msg->nesting_level] = nla;
+	msg->nests[msg->nesting_level].type = type;
+	msg->nests[msg->nesting_level].offset = msg->len;
 	msg->nesting_level += 1;
 
 	msg->len += NLA_HDRLEN;
@@ -864,8 +861,9 @@ LIB_EXPORT bool l_genl_msg_leave_nested(struct l_genl_msg *msg)
 	if (unlikely(msg->nesting_level == 0))
 		return false;
 
-	nla = msg->nests[msg->nesting_level - 1];
-	nla->nla_len = msg->len - nla->nla_len;
+	nla = msg->data + msg->nests[msg->nesting_level - 1].offset;
+	nla->nla_type = msg->nests[msg->nesting_level - 1].type;
+	nla->nla_len = msg->len - msg->nests[msg->nesting_level - 1].offset;
 
 	msg->nesting_level -= 1;
 
@@ -1147,9 +1145,46 @@ LIB_EXPORT void l_genl_family_unref(struct l_genl_family *family)
 	if (family->watch_destroy)
 		family->watch_destroy(family->watch_data);
 
+	l_genl_family_set_unicast_handler(family, NULL, NULL, NULL);
+
 	l_free(family);
 }
 
+LIB_EXPORT bool l_genl_family_set_unicast_handler(struct l_genl_family *family,
+						l_genl_msg_func_t handler,
+						void *user_data,
+						l_genl_destroy_func_t destroy)
+{
+	struct genl_unicast_notify *notify;
+
+	if (!family)
+		return false;
+
+	notify = family->unicast_notify;
+	if (notify) {
+		if (notify->destroy)
+			notify->destroy(notify->user_data);
+
+		if (!handler) {
+			l_free(notify);
+			family->unicast_notify = NULL;
+			return true;
+		}
+	} else {
+		if (!handler)
+			return false;
+
+		notify = l_new(struct genl_unicast_notify, 1);
+		family->unicast_notify = notify;
+	}
+
+	notify->handler = handler;
+	notify->destroy = destroy;
+	notify->user_data = user_data;
+
+	return true;
+}
+
 LIB_EXPORT bool l_genl_family_set_watches(struct l_genl_family *family,
 						l_genl_watch_func_t appeared,
 						l_genl_watch_func_t vanished,
