@@ -35,8 +35,7 @@
 
 #include <ell/ell.h>
 
-static void signal_handler(struct l_signal *signal, uint32_t signo,
-							void *user_data)
+static void signal_handler(uint32_t signo, void *user_data)
 {
 	switch (signo) {
 	case SIGINT:
@@ -69,8 +68,6 @@ static void event_handler(struct l_dhcp_client *client,
 int main(int argc, char *argv[])
 {
 	struct l_dhcp_client *client;
-	struct l_signal *signal;
-	sigset_t mask;
 	int ifindex;
 	uint8_t mac[6];
 
@@ -89,11 +86,6 @@ int main(int argc, char *argv[])
 	if (!l_main_init())
 		return -1;
 
-	sigemptyset(&mask);
-	sigaddset(&mask, SIGINT);
-	sigaddset(&mask, SIGTERM);
-	signal = l_signal_create(&mask, signal_handler, NULL, NULL);
-
 	l_log_set_stderr();
 	l_debug_enable("*");
 
@@ -102,10 +94,9 @@ int main(int argc, char *argv[])
 	l_dhcp_client_set_event_handler(client, event_handler, NULL, NULL);
 	l_dhcp_client_start(client);
 
-	l_main_run();
+	l_main_run_with_signal(signal_handler, NULL);
 
 	l_dhcp_client_destroy(client);
-	l_signal_remove(signal);
 	l_main_exit();
 
 	return 0;
