@@ -29,9 +29,8 @@
 
 #include <ell/ell.h>
 
-#include "ell/cipher.h"
-#include "ell/checksum.h"
 #include "ell/tls-private.h"
+#include "ell/cert-private.h"
 
 static void test_tls10_prf(const void *data)
 {
@@ -211,30 +210,93 @@ static void test_tls12_prf(const void *data)
 	assert(!memcmp(out_buf, test->expected, test->out_len));
 }
 
+static struct l_cert *load_cert_file(const char *filename)
+{
+	uint8_t *der;
+	size_t len;
+	char *label;
+	struct l_cert *cert;
+
+	der = l_pem_load_file(filename, 0, &label, &len);
+	if (!der)
+		return NULL;
+
+	l_free(label);
+	cert = l_cert_new_from_der(der, len);
+	l_free(der);
+	return cert;
+}
+
 static void test_certificates(const void *data)
 {
-	struct tls_cert *cert;
-	struct tls_cert *cacert;
-	struct tls_cert *wrongca;
+	struct l_queue *cacert;
+	struct l_queue *wrongca;
+	struct l_queue *twocas;
+	struct l_certchain *chain;
+	struct l_certchain *chain2;
+	struct l_certchain *chain3;
+	struct l_certchain *chain4;
 
-	cert = tls_cert_load_file(CERTDIR "cert-server.pem");
-	assert(cert);
+	cacert = l_pem_load_certificate_list(CERTDIR "cert-ca.pem");
+	assert(cacert && !l_queue_isempty(cacert));
 
-	cacert = tls_cert_load_file(CERTDIR "cert-ca.pem");
-	assert(cacert);
+	wrongca = l_pem_load_certificate_list(CERTDIR "cert-intca.pem");
+	assert(wrongca && !l_queue_isempty(wrongca));
 
-	wrongca = tls_cert_load_file(CERTDIR "cert-intca.pem");
-	assert(wrongca);
+	twocas = l_pem_load_certificate_list(CERTDIR "cert-chain.pem");
+	assert(twocas);
 
-	assert(!tls_cert_verify_certchain(cert, wrongca));
+	chain = l_pem_load_certificate_chain(CERTDIR "cert-server.pem");
+	assert(chain);
 
-	assert(tls_cert_verify_certchain(cert, cacert));
+	assert(!l_certchain_verify(chain, wrongca, NULL));
+	assert(l_certchain_verify(chain, cacert, NULL));
+	assert(l_certchain_verify(chain, NULL, NULL));
+	assert(l_certchain_verify(chain, twocas, NULL));
 
-	assert(tls_cert_verify_certchain(cert, NULL));
+	chain2 = l_pem_load_certificate_chain(CERTDIR "cert-chain.pem");
+	assert(chain2);
 
-	l_free(cert);
-	l_free(cacert);
-	l_free(wrongca);
+	assert(!l_certchain_verify(chain2, wrongca, NULL));
+	assert(l_certchain_verify(chain2, cacert, NULL));
+	assert(l_certchain_verify(chain2, NULL, NULL));
+	assert(l_certchain_verify(chain2, twocas, NULL));
+
+	chain3 = certchain_new_from_leaf(
+			load_cert_file(CERTDIR "cert-server.pem"));
+	certchain_link_issuer(chain3,
+			load_cert_file(CERTDIR "cert-entity-int.pem"));
+	certchain_link_issuer(chain3,
+			load_cert_file(CERTDIR "cert-intca.pem"));
+	certchain_link_issuer(chain3,
+			load_cert_file(CERTDIR "cert-ca.pem"));
+	assert(chain3);
+
+	assert(!l_certchain_verify(chain3, wrongca, NULL));
+	assert(!l_certchain_verify(chain3, cacert, NULL));
+	assert(!l_certchain_verify(chain3, NULL, NULL));
+	assert(!l_certchain_verify(chain3, twocas, NULL));
+
+	chain4 = certchain_new_from_leaf(
+			load_cert_file(CERTDIR "cert-entity-int.pem"));
+	certchain_link_issuer(chain4,
+			load_cert_file(CERTDIR "cert-intca.pem"));
+	certchain_link_issuer(chain4,
+			load_cert_file(CERTDIR "cert-ca.pem"));
+	assert(chain4);
+
+	assert(!l_certchain_verify(chain4, wrongca, NULL));
+	assert(l_certchain_verify(chain4, cacert, NULL));
+	assert(l_certchain_verify(chain4, NULL, NULL));
+	assert(l_certchain_verify(chain4, twocas, NULL));
+
+	l_certchain_free(chain);
+	l_certchain_free(chain2);
+	l_certchain_free(chain3);
+	l_certchain_free(chain4);
+	l_queue_destroy(cacert, (l_queue_destroy_func_t) l_cert_free);
+	l_queue_destroy(wrongca, (l_queue_destroy_func_t) l_cert_free);
+	l_queue_destroy(twocas, (l_queue_destroy_func_t) l_cert_free);
 }
 
 struct tls_conn_test {
@@ -352,6 +414,9 @@ static void tls_test_ready(const char *peer_identity, void *user_data)
 	struct tls_test_state *s = user_data;
 
 	assert(!s->ready);
+	assert((!s->expect_peer && !peer_identity) ||
+			(s->expect_peer && peer_identity &&
+			 !strcmp(s->expect_peer, peer_identity)));
 	s->ready = true;
 
 	l_tls_write(s->tls, (const uint8_t *) s->send_data,
@@ -418,8 +483,11 @@ static void test_tls_test(const void *data)
 					test->client_key_path,
 					test->client_key_passphrase);
 	assert(auth_ok);
-	l_tls_set_cacert(s[0].tls, test->server_ca_cert_path);
-	l_tls_set_cacert(s[1].tls, test->client_ca_cert_path);
+	assert(l_tls_set_cacert(s[0].tls, test->server_ca_cert_path));
+	assert(l_tls_set_cacert(s[1].tls, test->client_ca_cert_path));
+
+	assert(l_tls_start(s[0].tls));
+	assert(l_tls_start(s[1].tls));
 
 	while (1) {
 		if (s[0].raw_buf_len) {
